PROGRAM main
  IMPLICIT NONE
  CHARACTER*512                   :: config
  CHARACTER*50                    :: participantName, meshName, writeInitialData, readItCheckp, writeItCheckp
  CHARACTER*50                    :: readDataName, writeDataName
  INTEGER                         :: rank, commsize, ongoing, dimensions, meshID, bool, numberOfVertices, i,j
  INTEGER                         :: readDataID, writeDataID
  DOUBLE PRECISION                :: dt
  DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: vertices, writeData, readData
  DOUBLE PRECISION, DIMENSION(:)::boundingBox(6)
  INTEGER, DIMENSION(:), ALLOCATABLE :: vertexIDs

  ! Constants in f90 have to be prefilled with blanks to be compatible with preCICE
  writeInitialData(1:50)='                                                  '
  readItCheckp(1:50)='                                                  '
  writeItCheckp(1:50)='                                                  '

  CALL precicef_action_write_initial_data(writeInitialData)
  CALL precicef_action_read_iter_checkp(readItCheckp)
  CALL precicef_action_write_iter_checkp(writeItCheckp)

  WRITE (*,*) 'DUMMY: Starting Fortran solver dummy...'
  
  config = 'precice-config.xml'
  participantName = 'Fluid'
  meshName = "Solid-Mesh"
  WriteDataName = "Force"



  rank = 0
  commsize = 1
  dt = 1
  numberOfVertices = 3
  WRITE (*,*) 'Before precicef_create'
  CALL precicef_create(participantName, config, rank, commsize)
  WRITE (*,*) ' after precicef_create'

  CALL precicef_get_dims(dimensions)
  CALL precicef_get_mesh_id(meshName, meshID)
  CALL precicef_get_data_id(writeDataName,meshID,writeDataID)


  boundingBox(1) = 0.0d0 ! lower x bound
  boundingBox(2) = 4.0d0 ! upper x bound
  boundingBox(3) = 0.0d0 ! lower y bound
  boundingBox(4) = 4.0d0 ! upper y bound
  boundingBox(5) = 0.0d0 ! lower y bound
  boundingBox(6) = 4.0d0 ! upper y bound

  ! Allocate dummy mesh with only one vertex
  ! CALL precicef_set_mesh_access_region(0, boundingBox)
  CALL precicef_set_mesh_access_region(meshID, boundingBox)

  WRITE (*,*) 'before precicef_initialize'
  CALL precicef_initialize(dt)
  WRITE (*,*) 'after precicef_initialize'

  CALL precicef_initialize_data()

  ! const int otherMeshSize = precice.getMeshVertexSize(otherMeshID);返回网格的顶点数。
  CALL precicef_get_mesh_vertex_size(meshID,numberOfVertices)



  ALLOCATE(vertices(dimensions*numberOfVertices))
  ALLOCATE(vertexIDs(numberOfVertices))
  ALLOCATE(writeData(numberOfVertices*dimensions))

  DO i = 1, numberOfVertices
    vertexIDs(i) = 0 ! 或者其他初始值
  END DO

  DO i = 1, numberOfVertices*dimensions
    vertices(i) = 0.0d0 ! 或者其他初始值
    writeData(i) = 0.0d0 ! 或者其他初始值
  END DO

  CALL precicef_get_mesh_vertices_and_IDs(meshID, numberOfVertices, vertexIDs, vertices)



  CALL precicef_is_coupling_ongoing(ongoing)
  DO WHILE (ongoing.NE.0)
  
    CALL precicef_is_action_required(writeItCheckp, bool)
    
    IF (bool.EQ.1) THEN
      WRITE (*,*) 'DUMMY: Writing iteration checkpoint'
      CALL precicef_mark_action_fulfilled(writeItCheckp)
    ENDIF



    WRITE (*,*) 'readData: ', vertices

    writeData = vertices 

    CALL precicef_is_write_data_required(dt, bool)
    IF (bool.EQ.1) THEN
      CALL precicef_write_bvdata(writeDataID, numberOfVertices, vertexIDs, writeData)
    ENDIF

    CALL precicef_advance(dt)
    CALL precicef_is_coupling_ongoing(ongoing)

  ENDDO

  CALL precicef_finalize()
  WRITE (*,*) 'DUMMY: Closing Fortran solver dummy...'

  DEALLOCATE(writeData)
  DEALLOCATE(vertices)
  DEALLOCATE(vertexIDs)

END PROGRAM
